// $Id: EAC2iTunes.js 103 2007-02-07 22:16:22Z cjk32 $
// Copyright 2007 Christoper Key (cjk32@cam.ac.uk)

var EAC2iTunesRev = String("$Rev: 103 $").match(/\d+/);
var EAC2iTunesVersion = "0.5.0." + EAC2iTunesRev;
var ImportedBy = "Imported by EAC2iTunes " + EAC2iTunesVersion;

/* Script Options*/
var OptionsFile = "C:\\Windows\\EAC2iTunes.txt"

var SilentRetry;
var AddImportedByToComments;
var AddReadErrorsToComments;
var DeleteTrackAfterImport;

var EACExePath;
var EACExtractPath ;

var AlbumLogFilePath;
var AppendToAlbumLogFile;


/* Constants */
var ERR_COULD_NOT_CREATE_OBJECT = -2146959355;
var ERR_INVALID_POINTER = -2147467261;
var ERR_CALL_REJECTED_BY_CALLEE = -2147418111;
var ERR_TRACK_DELETED = -1610350078;

// TODO: Which values won't cause clashes?
var ERR_CANNOT_FIND_TRACKNO = -1;
var ERR_FAILED_TO_ADD_TRACK = -2;
var ERR_FAILED_TO_CONVERT_TRACK = -3;

var EACLogFileName = "Unknown Title.log";
var EACLogDivider = "------------------------------------------------------------";
var CRLF = "\r\n";

var ITSourceKindAudioCD = 3;

var ForReading = 1;
var ForWriting = 2;
var ForAppending = 8;

var vbOKOnly = 0;
var vbOKCancel = 1;
var vbAbortRetryIgnore = 2;
var vbYesNoCancel = 3;
var vbYesNo = 4;
var vbRetryCancel = 5;

var vbCritical = 16;
var vbQuestion = 32;
var vbExclamation = 48;
var vbInformation = 64;

var vbOK = 1;
var vbCancel = 2;
var vbAbort = 3;
var vbRetry = 4;
var vbIgnore = 5;
var vbYes = 6;
var vbNo = 7;

/* Main Program*/

/* Create utility objects */
var FSO = WScript.CreateObject("Scripting.FileSystemObject");
var Shell = WScript.CreateObject('WScript.Shell');
var commonDialog = WScript.CreateObject("UserAccounts.CommonDialog");

var args = WScript.Arguments;

if (args.Length == 0) {
	/* Read or create options file */
	if (!getOptions(OptionsFile)){ WScript.Quit(1); }

	/* Create iTunesApp object */
	var iTunesApp = getiTunesAppObject();

	/* Get the iTunes main library */
	var mainLibrary = iTunesMainLibrary(iTunesApp);

	/* Get the cdSource */
	var cdSource;
	if (!(cdSource = iTunesCDSource(iTunesApp))){ WScript.Quit(1) }

	/* Browse for artwork to use for the album */
	var artworkFile = "";

//	Why does the following fail?
//	commonDialog.DialogTitle = "Select Artwork";
	commonDialog.Filter = "Image Files|*.bmp;*.gif;*.jpg;*.png;|All Files|*.*|";

	if (commonDialog.ShowOpen()) {
		artworkFile = commonDialog.FileName;
	}

	/* Get the last modified date for the logfile if it exists */
	var logFilePath = FSO.BuildPath(EACExtractPath, EACLogFileName);
	var logFileModified = "";
	if (FSO.FileExists(logFilePath)) {
		logFileModified = FSO.GetFile(logFilePath).DateLastModified;
	}

	/* Run EAC in auto mode to extract all the contents of the 
	   CD.  If the users wants to manually select tracks, they 
	   can interrupt EAC when it starts. */

	Shell.Run('"' + EACExePath + '"' + ' -extract -close', 1, true);

	/* Check that EAC wrote a log file */
	if (!FSO.FileExists(logFilePath)){
		WScript.Echo("Unable to find log file.");
		WScript.Quit(1);
	}
	/* Check that the log file's date changed */
	logFile = FSO.GetFile(logFilePath);

	if (String(logFileModified) == String(logFile.DateLastModified)) { // == doesn't work in date objects
		if (Shell.Popup("EAC log file doesn't appear to have changed, continue anyway?",0,"EAC Log File Unchanged",vbYesNo) == vbNo) {
			WScript.Quit(1);
		}
	}

	/* Read the last segment of the EAC log file */
	logFile = FSO.OpenTextFile(logFilePath, ForReading, false);

	var cdLogData = "";
	while (true) {  //logFile.AtEndOfSteam does seem to work for some reason
		try {
			var line = logFile.ReadLine();
		}
		catch (e) {
			break;
		}

		if (line == EACLogDivider) {
			cdLogData = "";
			logFile.SkipLine(); // Skip blank line that follows the seperator
		}else{
			cdLogData += line + CRLF;
		}
	}

	/* Save out this section of the log file under an appropriate name */
	var cdDescription = cdSourceDesc(cdSource);
	cdLogData = cdLogData.replace(/^Unknown Artist \/ Unknown Title$/m, cdDescription); //

	// Product a version of the title suibale for use as a filename
	cdDescription = cdDescription.replace(/\s*[\/\\\:\*\?\"\<\>\|]+\s*/g,"_");

	if (AlbumLogFilePath) {
		if (!FSO.FolderExists(AlbumLogFilePath)) { FSO.CreateFolder(AlbumLogFilePath) }

		// Loop until we find a unique file name unless we're going to append to the file
		var logFileNo = "";
		do {
			logFile = FSO.BuildPath(AlbumLogFilePath, cdDescription + logFileNo + ".log");
			if (logFileNo == "") {
				logFileNo = " 1";
			}else{
				logFileNo = " " + (parseInt(logFileNo) + 1);
			}
		} while (FSO.FileExists(logFile) && !AppendToAlbumLogFile)

		// If the file exists, append a divider, else just create the file
		if (FSO.FileExists(logFile)) {
			logFile = FSO.OpenTextFile(logFile, ForAppending, false);
			logFile.Write(CRLF + EACLogDivider + CRLF + CRLF);
		} else {
			logFile = FSO.CreateTextFile(logFile, false);
		}

		logFile.Write(cdLogData);
		logFile.Close();
	}

	/* Extract an array of the reports for each track.
	   The regexp matchs a line starting with Track nn, then all 
	   subsequent lines that are either blank, or start with whitespace.
	  */
	var trackLogData = cdLogData.match(/^Track\s+\d+\r\n((|\s+[^\r\n]*)\r\n)*/mg);

	/* Loop through the tracks and import into iTunes */
track:	for (var i=0; i < trackLogData.length; i++) {
		var trackNo = trackLogData[i].match(/^Track\s+(\d+)/m)[1];
		var fileName = trackLogData[i].match(/^\s+Filename\s+([^\r\n]*)/m)[1];
		var readErrors = trackLogData[i].match(/Suspicious position\s+[^\r\n]*/g);

		var addComments = "";
		if (readErrors && AddReadErrorsToComments) {addComments = readErrors.join("; ")}

		if (AddImportedByToComments) {
			addComments += (addComments ? CRLF : "") + ImportedBy;
		}

		while (true) {
			try {
				iTunesTagAndImport(iTunesApp, fileName, iTunesCDTrack(cdSource, trackNo), artworkFile, addComments, mainLibrary);
				break;
			}
			catch (e) {
				if (e.number == ERR_TRACK_DELETED) {
					if (!(cdSource = iTunesCDSource(iTunesApp))){ WScript.Quit(1) }
				} else {
					var response = Shell.Popup("Failed To Add Track:"+ CRLF + e.description,0,"Failed To Add Track",vbAbortRetryIgnore)
					if (response == vbAbort) {
						WScript.Quit(1);
					} else if (response == vbIgnore) {
						continue track;
					} // else loop and retry
				}
			}
		}

		if (DeleteTrackAfterImport) {
			FSO.DeleteFile(fileName);
		}
	}

	WScript.Echo("Import Complete");
	WScript.Quit(1);

} else if (args.Length == 2) {

	var trackNo = args.item(0);
	var fileName = trim(args.item(1));

	/* Create iTunesApp object */
	var iTunesApp = getiTunesAppObject();

	/* Import the file */
	iTunesTagAndImport(iTunesApp, fileName, iTunesCDTrack(iTunesCDSource(iTunesApp), trackNo), "", "", iTunesMainLibrary(iTunesApp))

}else{
	WScript.Echo("Usage: [cscript | wscript] EAC2iTunes.vbs <trackno> <filename>");
	WScript.Quit(1);
}

function getOptions(optionsFileName) {
	var optionsFile;
	var optionsFileData = "";
	if (FSO.FileExists(optionsFileName)) {
		optionsFile = FSO.OpenTextFile(optionsFileName, ForReading, false)
		optionsFileData = optionsFile.ReadAll();
		optionsFile.Close();
	}else{
		Shell.Popup("EAC2iTunes needs to be configured");
	}

	SilentRetry = getOptionValue(optionsFileData, "SilentRetry", true);

	if ((EACExePath = getOptionValue(optionsFileData, "EACExePath", null)) == null) {
		Shell.Popup("Please locate the Exact Audio Copy executable.");
		commonDialog.Filter = "EAC.exe|EAC.exe";
		commonDialog.Filename = "EAC.exe";
		//TODO Common Dialog properties
		if (commonDialog.ShowOpen()){
			EACExePath = commonDialog.FileName;
		} else {
			return false;
		}
	}

	if ((EACExtractPath = getOptionValue(optionsFileData, "EACExtractPath", null)) == null) {
		commonDialog.Filter = "Filename Ignored|*._";
		commonDialog.Filename = "Filename Ignored";
		Shell.Popup("Please locate the folder that EAC extracts tracks to.");
		if (commonDialog.ShowOpen()){
			EACExtractPath = FSO.GetParentFolderName(commonDialog.FileName);
		} else {
			return false;
		}
	}

	if ((AlbumLogFilePath = getOptionValue(optionsFileData, "AlbumLogFilePath", null)) == null) {
		commonDialog.Filter = "Filename Ignored|*._";
		commonDialog.Filename = "Filename Ignored";
		Shell.Popup("Please select a folder to save the extract logs to.");
		if (commonDialog.ShowOpen()){
			AlbumLogFilePath = FSO.GetParentFolderName(commonDialog.FileName);
		} else {
			AlbumLogFilePath = false;
		}
	}

	if ((AddImportedByToComments = getOptionValue(optionsFileData, "AddImportedByToComments", null)) == null) {
		AddImportedByToComments = 
			(Shell.Popup("Do you want EAC2iTunes to add a version line to the track comments?"
			,0,"Add Imported By To Comments",vbYesNo) == vbYes);
	}

	if ((AddReadErrorsToComments = getOptionValue(optionsFileData, "AddReadErrorsToComments", null)) == null) {
		AddReadErrorsToComments = 
			(Shell.Popup("Do you want EAC2iTunes to add any ripping errors to the track comments?"
			,0,"Add Read Errors To Comments",vbYesNo) == vbYes);
	}

	if ((DeleteTrackAfterImport = getOptionValue(optionsFileData, "DeleteTrackAfterImport", null)) == null) {
		DeleteTrackAfterImport = 
			(Shell.Popup("Do you want EAC2iTunes to delete the WAV source file after importing the track?"
			,0,"Delete Track After Import",vbYesNo) == vbYes);
	}

	AppendToAlbumLogFile= getOptionValue(optionsFileData, "AppendToAlbumLogFile", true);


	optionsFileData = setOptionValue(optionsFileData, "SilentRetry",             SilentRetry);
	optionsFileData = setOptionValue(optionsFileData, "AddImportedByToComments", AddImportedByToComments);
	optionsFileData = setOptionValue(optionsFileData, "AddReadErrorsToComments", AddReadErrorsToComments);
	optionsFileData = setOptionValue(optionsFileData, "DeleteTrackAfterImport",  DeleteTrackAfterImport);
	optionsFileData = setOptionValue(optionsFileData, "EACExePath",              EACExePath);
	optionsFileData = setOptionValue(optionsFileData, "EACExtractPath",          EACExtractPath);
	optionsFileData = setOptionValue(optionsFileData, "AlbumLogFilePath",        AlbumLogFilePath);
	optionsFileData = setOptionValue(optionsFileData, "AppendToAlbumLogFile",    AppendToAlbumLogFile);

	optionsFile = FSO.OpenTextFile(optionsFileName, ForWriting, true)
	optionsFile.Write(optionsFileData);
	optionsFile.Close();

	return true;
}


function getOptionValue(optionsData, optionName, optionValue) {
	var readOptionValue = optionsData.match(new RegExp("^" + optionName + " : ([^\r\n]*)","m"));
	return readOptionValue ? readOptionValue[1] : optionValue;
}

function setOptionValue(optionsData, optionName, optionValue) {
	var re = new RegExp("^" + optionName + " : [^\r\n]*","m");
	if (optionsData.match(re)) {
		optionsData = optionsData.replace(re, optionName + " : " + optionValue);
	} else {
		if (optionsData.match(/.+[^\r\n]$/)) {
			optionsData = optionsData + CRLF;
		}
		optionsData = optionsData + optionName + " : " + optionValue + CRLF;
	}
	return optionsData;
}

function getiTunesAppObject() {
	do {
		try {
			return WScript.CreateObject("iTunes.Application");
			break;
		}
		catch (e) {
			if (e.number == ERR_COULD_NOT_CREATE_OBJECT){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)
}


function iTunesMainLibrary(iTunesApp) {
	do {
		try {
			return iTunesApp.LibraryPlaylist;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)
}

/* Return a reference to an iTunes CD source
   If there are multiple sources, prompt the user 
   to select one */

function iTunesCDSource(iTunesApp) {
	/* Count the number of iTunes sources of kind AudioCD */
outer:	while (true) {
		while (true) {
			var cdSourceCount = 0;
			var cdSourceIDs = new Array();
			var cdSourceNames = new Array();

			try {
				for (var i = 1; i <= iTunesApp.Sources.Count; i++) {
					if (iTunesApp.Sources.Item(i).Kind == ITSourceKindAudioCD) {
						cdSourceCount++;
						cdSourceIDs[cdSourceCount-1] = iTunesApp.Sources.Item(i).sourceID;
						cdSourceNames[cdSourceCount-1] = iTunesApp.Sources.Item(i).Name;
					}
				}

				break;
			}
			catch (e) {
				if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
			}

			WScript.Sleep(100);

		}

		if (cdSourceCount == 0) {

			if (Shell.Popup("No CD sources were found in iTunes, please (re)insert the CD.",0,"No CD Source Found",vbRetryCancel) == vbCancel) {
				return null;
			} else {
				continue outer;
			}
		} else if (cdSourceCount == 1) {
			var cdSourceID = cdSourceIDs[0];
		} else {
			// TODO get the user to select a device
		}

		while (true) {
			try {
				for (var i = 1; i <= iTunesApp.Sources.Count; i++) {
					if (iTunesApp.Sources.Item(i).sourceID == cdSourceID) return iTunesApp.Sources.item(i);
				}
				

				if (Shell.Popup("Unable to find CD source.",0,"Unable to find CD source",vbRetryCancel) == vbCancel) {
					return null;
				} else {
					continue outer; // Not required, but make the loop structure a bit clearer
				}

			}
			catch (e) {
				if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
			}

			WScript.Sleep(100);
		}
	}
}

function cdSourceDesc(cdSource) {
	do {
		try {
			var playlist = cdSource.Playlists.Item(1);
			var desc = "";

			if (playlist.Compilation) {
				desc = "Various Artists";
			} else {
				desc = playlist.Artist;
			}

			desc += " / " + playlist.Name;

			if (playlist.DiscCount > 1) {
				desc += " Disc " + playlist.DiscNumber + " of " + playlist.DiscCount;
			}

			return desc;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)
}

function iTunesCDTrack(cdSource, trackNo) {
	do {
		try {
			for (var i = 1; i <= cdSource.Playlists.Item(1).Tracks.Count; i++) {
				var cdTrack = cdSource.Playlists.Item(1).Tracks.Item(i);
				if (cdTrack.TrackNumber == trackNo) return cdTrack;
			}

			throw new Error(ERR_CANNOT_FIND_TRACKNO,"Unable to find track " + trackNo);
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)
}


function iTunesTagAndImport(iTunesApp, fileName, tagsTrack, artworkPath, addComments, library) {

	/* Add track to the library */

	do {
		try {
			var addStatus = library.addFile(fileName);

			if (!addStatus) {
				throw new Error(ERR_FAILED_TO_ADD_TRACK,"Failed to add track '" + fileName + "'");
			}

			break;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)


	/* Wait for operation to complete and get a handle to the added track*/

	do {
		try {
			do {
				WScript.Sleep(100);
			} while (addStatus.inProgress)

			var addedTrack = addStatus.Tracks.Item(1);

			break;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)


	/* Copy across tags from the source track to the newly added track */

	/* ERR_INVALID_POINTER occurs with empty strings for some reason.  Ignore these and carry on */

	do {
		try {
			// The following is a complete list as of iTunes 1.8 type library
			try {addedTrack.Album              = tagsTrack.Album              } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.AlbumArtist        = tagsTrack.AlbumArtist        } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Artist             = tagsTrack.Artist             } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
	//		try {addedTrack.BookmarkTime       = tagsTrack.BookmarkTime       } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.BPM                = tagsTrack.BPM                } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Category           = tagsTrack.Category           } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Comment            = tagsTrack.Comment            } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Compilation        = tagsTrack.Compilation        } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Composer           = tagsTrack.Composer           } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Description        = tagsTrack.Description        } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.DiscCount          = tagsTrack.DiscCount          } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.DiscNumber         = tagsTrack.DiscNumber         } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Enabled            = tagsTrack.Enabled            } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.EpisodeID          = tagsTrack.EpisodeID          } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.EpisodeNumber      = tagsTrack.EpisodeNumber      } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.EQ                 = tagsTrack.EQ                 } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.ExcludeFromShuffle = tagsTrack.ExcludeFromShuffle } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Finish             = tagsTrack.Finish             } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Genre              = tagsTrack.Genre              } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Grouping           = tagsTrack.Grouping           } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.LongDescription    = tagsTrack.LongDescription    } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
	//		try {addedTrack.Lyrics             = tagsTrack.Lyrics             } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Name               = tagsTrack.Name               } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.PartOfGaplessAlbum = tagsTrack.PartOfGaplessAlbum } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.PlayedCount        = tagsTrack.PlayedCount        } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.PlayedDate         = tagsTrack.PlayedDate         } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Rating             = tagsTrack.Rating             } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.RememberBookmark   = tagsTrack.RememberBookmark   } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.SeasonNumber       = tagsTrack.SeasonNumber       } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Show               = tagsTrack.Show               } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.SkippedCount       = tagsTrack.SkippedCount       } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.SkippedDate        = tagsTrack.SkippedDate        } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Start              = tagsTrack.Start              } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.TrackCount         = tagsTrack.TrackCount         } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.TrackNumber        = tagsTrack.TrackNumber        } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
	//		try {addedTrack.VideoKind          = tagsTrack.VideoKind          } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.VolumeAdjustment   = tagsTrack.VolumeAdjustment   } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}
			try {addedTrack.Year               = tagsTrack.Year               } catch (e) {if (e.number != ERR_INVALID_POINTER){ throw e }}

			if (addComments != "") {
				if (addedTrack.Comment != "") { addedTrack.Comment += CRLF }
				addedTrack.Comment += addComments;
			}

			break;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)


	/* Convert new track to default audio format */

	do {
		try {
			var convertStatus = iTunesApp.ConvertTrack2(addedTrack);

			if (!convertStatus) {
				throw new Error(ERR_FAILED_TO_CONVERT_TRACK,"Failed to convert track '" + newTrack.name + "'.");

			}

			break;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)


	/* Wait for completion and then get a handle to the converted track */

	do {
		try {
			do {
				WScript.Sleep(100);
			} while (convertStatus.inProgress)

			var convertedTrack = convertStatus.Tracks.Item(1);

			break;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)


	/* Deleted the unconverted track */

	do {
		try {
			addedTrack.Delete();

			break;
		}
		catch (e) {
			if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
		}

		WScript.Sleep(100);

	} while (true)


	/* If artwork was supplied, add it to the converted track */

	if (artworkPath) {
		do {
			try {
				convertedTrack.AddArtworkFromFile(artworkPath);

				break;
			}
			catch (e) {
				if (e.number == ERR_CALL_REJECTED_BY_CALLEE){ iTunesError(e) }else{ throw(e) }
			}

			WScript.Sleep(100);

		} while (true)
	}
}

function iTunesError(e) {
	if (!SilentRetry) WScript.echo("iTunes error: " + e.number + CRLF + e.description + CRLF + CRLF + "Please close all open dialogs in iTunes.");
}

function trim (str) { return str.replace(/^\s+/g, '').replace(/\s+$/g, ''); }
